/**
 * Copyright (c) 2003, James d'Arcy <james.darcy@wraithbane.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of wraithbane.com nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

public class DisplayCell extends JLayeredPane
{
    
    public static final Integer IMAGE_LAYER = new Integer(1);
    public static final Integer ANNOTATION_LAYER = new Integer(10);
    public static final Integer ROI_LAYER = new Integer(2);
    
    protected ImageCell _imgCell;
    protected ImageAnnotation _imgAnnotation;
    
    protected Color _active = Color.red;
    protected Color _inactive = Color.black;
    protected boolean _isActive;
    protected Image _img;
    protected short[] _pixels;
    protected byte[] _bPixBuffer;
    protected int _winWidth, _winCentre;
    protected ColorModel _cm;
    protected int _imgWidth, _imgHeight;
    protected int _numPixels;
    protected int _mouseX, _mouseY;
    protected boolean _lmbDown, _mmbDown, _rmbDown;
    
    /** Creates a new instance of DisplayCell */
    public DisplayCell()
    {
        Dimension dims;
        
        ImageCellMouseInputAdapter adapter = new ImageCellMouseInputAdapter();
        
        initCM();
        
        addMouseListener(adapter);
        addMouseMotionListener(adapter);
        
        _imgCell = new ImageCell();
        _imgAnnotation = new ImageAnnotation();
        
        add(_imgCell, IMAGE_LAYER);
        dims = getPreferredSize();
        _imgCell.setBounds(2, 2, dims.width-4, dims.height-4);
        add(_imgAnnotation, ANNOTATION_LAYER);
        _imgAnnotation.setBounds(2, 2, dims.width-4, dims.height-4);

        setBackground(Color.black);
        setBorder(new LineBorder(_inactive, 2));
        
        setWindowLevel(256, 512, false);
    }
        
    class ImageCellMouseInputAdapter extends MouseInputAdapter
    {
        public void mouseDragged(MouseEvent e)
        {
            int mX = e.getX();
            int mY = e.getY();
            if (((e.getModifiers() & InputEvent.BUTTON2_MASK) != 0) && 
                _mmbDown && _isActive)
            {
                incrementWindowLevel((mX-_mouseX),(mY-_mouseY));
                applyWindowLevel();
                _mouseX = mX;
                _mouseY = mY;
                repaint();
            }
        }

        public void mouseClicked(MouseEvent e)
        {
            setActive();
        }
        
        public void mousePressed(MouseEvent e)
        {
            if ((e.getModifiers() & InputEvent.BUTTON2_MASK) != 0)
            {
                _mouseX = e.getX();
                _mouseY = e.getY();
                _mmbDown = true;
            }
        }
        
        public void mouseReleased(MouseEvent e)
        {
            if ((e.getModifiers() & InputEvent.BUTTON2_MASK) != 0)
            {
                _mmbDown = false;
            }
        }
    
    }
    
    protected void applyWindowLevel()
    {
        int i, winMax, winMin;
        
        winMin = _winCentre-_winWidth/2;
        winMax = winMin+_winWidth;
        
        for (i=0; i<(_numPixels); i++)
        {
            _bPixBuffer[i] = (byte)(255*(Math.max(Math.min(_pixels[i],winMax),
                winMin)-winMin)/(winMax-winMin));
        }
        
        _imgCell.setImage(createImage(new MemoryImageSource(_imgWidth, _imgHeight, 
            _cm, _bPixBuffer, 0, _imgWidth)));
        repaint();
    }
    
    public Dimension getMinimumSize()
    {
        Dimension dims = _imgCell.getMinimumSize();
        return new Dimension(dims.width+4, dims.height+4);
    }
    
    public Dimension getPreferredSize()
    {
        Dimension dims = _imgCell.getPreferredSize();
        return new Dimension(dims.width+4, dims.height+4);
    }
    
    protected void incrementWindowLevel(int incCentre, int incWidth)
    {
        _winCentre = (int)(Math.min(Math.max(_winCentre+incCentre, 1), 4094));
        _winWidth = (int)(Math.min(Math.max(_winWidth+incWidth, 1), 4094));
        _imgAnnotation.setWindowLevel(_winCentre, _winWidth);
    }
    
    protected void initCM()
    {
        byte[] ramp = new byte[256];
        for (int i=0; i<256; i++)
        {
            ramp[i] = (byte)i;
        }
        _cm = new IndexColorModel(8,256,ramp,ramp,ramp);
    }

    void setPixelData(short[] sData, int nX, int nY)
    {
        int i;
        
        _numPixels = nX*nY;
        _imgWidth = nX;
        _imgHeight = nY;
        _pixels = new short[_numPixels];
        _bPixBuffer = new byte[_numPixels];
        
        for (i=0; i<_numPixels; i++)
        {
            _pixels[i] = sData[i];
        }
        
        _imgAnnotation.setWindowLevel(_winCentre, _winWidth);
        applyWindowLevel();
        
        _imgAnnotation.setActive();
    }
    
    public void setActive()
    {
        setBorder(new LineBorder(_active, 2));
        _isActive = true;
    }
    
    public void setAcquisitionData(String tr, String te, String matrix,
        String fieldOfView)
    {
        _imgAnnotation.setAcquisitionData(tr, te, matrix, fieldOfView);
    }

    public void setPatientData(String patientName, String patientID, 
        String dateOfBirth)
    {
        _imgAnnotation.setPatientData(patientName, patientID, dateOfBirth);
    }
    
    public void setSeriesData(String seriesDescr, String studyDate, 
        String seriesTime, String seriesNum, String imaNum)
    {
        _imgAnnotation.setSeriesData(seriesDescr, studyDate, seriesTime, 
            seriesNum, imaNum);
    }
    
    public void setSpatialData(String sliceThick, String slicePos, 
        String sliceOrient)
    {
        _imgAnnotation.setSpatialData(sliceThick, slicePos, sliceOrient);
    }
    
    public void setInactive()
    {
        setBorder(new LineBorder(_inactive, 2));
        _isActive = false;
    }
    
    public void setWindowLevel(int centre, int width)
    {
        _winCentre = (int)(Math.min(Math.max(centre, 1), 4094));
        _winWidth = (int)(Math.min(Math.max(width, 1), 4094));
        _imgAnnotation.setWindowLevel(_winCentre, _winWidth);
        applyWindowLevel();
    }
    
    public void setWindowLevel(int centre, int width, boolean apply)
    {
        _winCentre = (int)(Math.min(Math.max(centre, 1), 4094));
        _winWidth = (int)(Math.min(Math.max(width, 1), 4094));
        _imgAnnotation.setWindowLevel(_winCentre, _winWidth);
        if (apply)
        {
            applyWindowLevel();
        }
    }
    
}
