/**
 * Copyright (c) 2003, James d'Arcy <james.darcy@wraithbane.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of wraithbane.com nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.wraithbane.dicom;

import java.util.StringTokenizer;

public final class Dicom {
    
    public static final String IMP_VR_LITTLE_ENDIAN = "1.2.840.10008.1.2";
    public static final String EXP_VR_LITTLE_ENDIAN = "1.2.840.10008.1.2.1";
    public static final String EXP_VR_BIG_ENDIAN    = "1.2.840.10008.1.2.2";
    public static final int XFER_IMP_VR_LITTLE_ENDIAN = 0;
    public static final int XFER_EXP_VR_LITTLE_ENDIAN = 1;
    public static final int XFER_EXP_VR_BIG_ENDIAN    = 2;
    public static final String PREFIX               = "DICM";

    public static final String AE = "AE";
    public static final String AS = "AS";
    public static final String CS = "CS";
    public static final String DA = "DA";
    public static final String DS = "DS";
    public static final String DT = "DT";
    public static final String IS = "IS";
    public static final String LO = "LO";
    public static final String LT = "LT";
    public static final String PN = "PN";
    public static final String SH = "SH";
    public static final String ST = "ST";
    public static final String TM = "TM";
    public static final String UI = "UI";
    public static final String UT = "UT";
    public static final byte OB = 0;
    public static final byte SQ = 0;
    public static final byte UN = 0;
    public static final short OW = 0;
    public static final short SS = 0;
    public static final short US = 0;
    public static final int AT = 0;
    public static final int SL = 0;
    public static final int UL = 0;
    public static final float FL = 0.0f;
    public static final float OF = 0.0f;
    public static final double FD = 0.0;

    static final int _IMP_VR_LITTLE_ENDIAN = 0;
    static final int _EXP_VR_LITTLE_ENDIAN = 1;
    static final int _EXP_VR_BIG_ENDIAN    = 2;
    static final int _VR_UNKNOWN = 0;
    static final int PREAMBLE_LEN  = 128;
    static final int PREFIX_LEN = 4;

    /** Creates a new instance of Dicom */
    public Dicom()
    {
    }
    
    static short getInt16(byte[] bData, int iOffset)
    {
        short sReturn = 0;
        
        sReturn = (short) ((bData[iOffset+1] & 0xff) << 8);
        sReturn = (short) (sReturn | (bData[iOffset] & 0xff));
        
        return sReturn;
    }

    /* Get rid of pesky '\0's on the end of strings - all DICOM values have
     * even lengths */
    static String chomp(String string)
    {
        string.replace((char)0x0000, ' ');
        return string.trim();
    }
    
    static int getInt32(byte[] bData, int iOffset)
    {
        int iReturn = 0;
        
        iReturn = ((int) bData[iOffset+3] & 0xff) << 24;
        iReturn = iReturn | ((int) bData[iOffset+2] & 0xff) << 16;
        iReturn = iReturn | ((int) bData[iOffset+1] & 0xff) << 8;
        iReturn = iReturn | ((int) bData[iOffset] & 0xff);
        
        return iReturn;
    }

    static String getString(byte[] bData, int iOffset, int iLength)
    {
        char[] cString = new char[iLength];
        int i, j;
        
        for (i=iOffset, j=0; i<(iOffset+iLength); i++, j++)
        {
            cString[j] = (char)bData[i];
        }
        
        return new String(cString);
    }
    
    static String toHexString(short sData)
    {
        String strData = "0x";
        
        strData = strData + Integer.toHexString(0x10000 | sData).substring(1);
        
        return strData;
    }

    static String toHexString(int iData)
    {
        String strData = "0x";
        
        strData = strData + Integer.toHexString(0x10000 | (iData >> 16 & 0xffff)).substring(1);
        strData = strData + Integer.toHexString(0x10000 | (iData & 0xffff)).substring(1);
        
        return strData;
    }
    
    public static String prettifyDate(String strDate)
    {
        StringTokenizer tokenizer = new StringTokenizer(strDate, ".");
        String[] months = {" Jan ", " Feb ", " Mar ", " Apr ", " May ", " Jun ", 
            " Jul ", " Aug ", " Sep ", " Oct ", " Nov ", " Dec "};
        String[] tokens = new String[3];
        String strValue;
        int i;
        
        if (tokenizer.countTokens() != 3)
        {
            return strDate;
        }
        
        for (i=0; i<3; i++)
        {
            tokens[i] = tokenizer.nextToken();
        }
        strValue = tokens[2];
        strValue = strValue.concat(months[Integer.parseInt(tokens[1])]);
        strValue = strValue.concat(tokens[0]);
        
        return strValue;
    }
}
