/**
 * Copyright (c) 2003, James d'Arcy <james.darcy@wraithbane.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of wraithbane.com nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.wraithbane.dicom;

public class DicomElement
{
    String _VR;
    int _valueLen;
    int _elementLen;
    int _type;
    short _group;
    short _tag;
    int _offset;
    byte[] _value;
    
    /** Creates a new instance of DicomElement */
    public DicomElement(byte[] bData, int iOffset, int iSyntax)
    {
        _offset = iOffset;
        int iPos = iOffset;
        int i, j;
        short VR;
        
        // Get group and tag
        _group = Dicom.getInt16(bData, iPos);
        iPos += 2;
        _tag = Dicom.getInt16(bData, iPos);
        iPos += 2;
        
        switch (iSyntax)
        {
            case Dicom._IMP_VR_LITTLE_ENDIAN:
                // Get length of value
                _valueLen = Dicom.getInt32(bData, iPos);
                iPos += 4;
                _elementLen = _valueLen + 8;
                _value = new byte[_valueLen];
                // Copy data as byte array
                for (i=iPos, j=0; i<(iPos+_valueLen); i++, j++)
                {
                    _value[j] = bData[i];
                }
                break;

            case Dicom._EXP_VR_LITTLE_ENDIAN:
                // Get VR as string and short
                _VR = Dicom.getString(bData, iPos, 2);
                VR = Dicom.getInt16(bData, iPos);
                iPos += 2;
                // Get length of value
                _valueLen = Dicom.getInt16(bData, iPos);
                iPos += 2;
                /* Use short VR for switch (integer types only). Hex value is 
                 * reversed (eg. 4c 55 == LU) as little endian
                 */
                switch (VR)
                {
                    case 0x424f: // OB
                    case 0x574f: // OW
                    case 0x464f: // OF
                    case 0x5153: // SQ
                    case 0x4e55: // UN
                    case 0x5455: // UT
                        /* 2 bytes which went into _valueLen are a reserved
                         * field. Get actual value length now */
                        _valueLen = Dicom.getInt32(bData, iPos);
                        iPos += 4;
                        // Note that element is 4 bytes longer for these VR's
                        _elementLen = _valueLen+12;
                        break;

                    default:
                        _elementLen = _valueLen+8;
                }
                if (_valueLen < 0)
                {
                    System.out.println(Dicom.toHexString(iOffset));
                }
                _value = new byte[_valueLen];
                // Copy data as byte array
                for (i=iPos, j=0; i<(iPos+_valueLen); i++, j++)
                {
                    _value[j] = bData[i];
                }
                break;

            case Dicom._EXP_VR_BIG_ENDIAN:
                break;

            default:
        }
    }

    short getGroup()
    {
        return _group;
    }
    
    short[] getArrValue(short sType)
    {
        short [] arrValue;
        int i, j, arrLen;
        
        arrLen = _valueLen/2;
        arrValue = new short[arrLen];
        
        for (i=0,j=0; i<arrLen; i++,j+=2)
        {
            arrValue[i] = Dicom.getInt16(_value, j);
        }
        
        return arrValue;
    }
    
    public int getOffset()
    {
        return _offset;
    }
    
    byte getValue(byte bType)
    {
        return _value[0];
    }
    
    short getValue(short sType)
    {
        return Dicom.getInt16(_value, 0);
    }
    
    int getValue(int iType)
    {
        return Dicom.getInt32(_value, 0);
    }
    
    String getValue(String strType)
    {
        return Dicom.chomp(Dicom.getString(_value, 0, _valueLen));
    }
    
    public int getLength()
    {
        return _elementLen;
    }
    
    public int getValueLen()
    {
        return _valueLen;
    }
    
    short getTag()
    {
        return _tag;
    }
    
    String getVR()
    {
        return _VR;
    }
}
