/**
 * Copyright (c) 2003, James d'Arcy <james.darcy@wraithbane.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of wraithbane.com nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.wraithbane.dicom;

import java.io.*;
import java.util.Vector;

public class DicomFile {

    protected byte[] _preamble; // All zeroes for DICOM
    protected String _prefix;   // Must be 'DICM' for a valid file
    protected int _xferSyntax;  // Transfer Syntax
    protected int _dataLen;
    protected Vector _groups;
    
    protected String _fileName;
    protected String _filePath;

    /** Creates a new instance of DicomFile */
    public DicomFile()
    {
        _preamble = new byte[Dicom.PREAMBLE_LEN];
        
        _groups = new Vector(1, 1);
    }

    /** Creates a new instance of DicomFile */
    public DicomFile(String fileName) throws IOException
    {
        _preamble = new byte[Dicom.PREAMBLE_LEN];
        
        _groups = new Vector(1, 1);
        
        read(fileName);
    }

    public short[] getArrValue(int iGroup, int iElement, short sVR)
        throws DicomGroupNotFoundException, DicomElementNotFoundException
    {
        DicomGroup dcmGroup;
        int i, iGroups;
        short sGroup, sElement;
        short sValue[];
        
        sGroup = (short)iGroup;
        sElement = (short)iElement;
        
        iGroups = _groups.size();
        i = 0;
        dcmGroup = (DicomGroup)_groups.elementAt(i);
        while ((dcmGroup.group() != sGroup) && (++i < iGroups))
        {
            dcmGroup = (DicomGroup)_groups.elementAt(i);
        }
        if (i == iGroups)
        {
            throw new DicomGroupNotFoundException("Group "+Dicom.toHexString(sGroup)+" not found");
        }
        
        try
        {
            sValue = dcmGroup.getArrValue(sElement, sVR);
        }
        catch (DicomElementNotFoundException exDCM)
        {
            //sValue = -1;
            throw new DicomElementNotFoundException("Element "+Dicom.toHexString(sElement)+" not found");
        }
        
        return sValue;
    }

    public DicomGroup getGroup(int iGroup)
        throws DicomGroupNotFoundException
    {
        DicomGroup dcmGroup;
        int i, iGroups;
        short sGroup, sElement;
        int iValue;
        
        sGroup = (short)iGroup;
        
        iGroups = _groups.size();
        i = 0;
        dcmGroup = (DicomGroup)_groups.elementAt(i);
        while ((dcmGroup.group() != sGroup) && (++i < iGroups))
        {
            dcmGroup = (DicomGroup)_groups.elementAt(i);
        }
        if (i == iGroups)
        {
            throw new DicomGroupNotFoundException("Group "+Dicom.toHexString(sGroup)+" not found");
        }
        
        return dcmGroup;
    }

    public int getGroupLength(int iGroup)
        throws DicomGroupNotFoundException
    {
        DicomGroup dcmGroup;
        int i, iGroups;
        short sGroup, sElement;
        int iValue;
        
        sGroup = (short)iGroup;
        
        iGroups = _groups.size();
        i = 0;
        dcmGroup = (DicomGroup)_groups.elementAt(i);
        while ((dcmGroup.group() != sGroup) && (++i < iGroups))
        {
            dcmGroup = (DicomGroup)_groups.elementAt(i);
        }
        if (i == iGroups)
        {
            throw new DicomGroupNotFoundException("Group "+Dicom.toHexString(sGroup)+" not found");
        }
        
        iValue = dcmGroup.getLength();
        
        return iValue;
    }

    public int getOffsetToGroup(int iGroup)
        throws DicomGroupNotFoundException
    {
        DicomGroup dcmGroup;
        int i, iGroups;
        short sGroup, sElement;
        int iValue, iOffset;
        
        sGroup = (short)iGroup;
        iOffset = Dicom.PREAMBLE_LEN + Dicom.PREFIX_LEN;
        
        iGroups = _groups.size();
        i = 0;
        dcmGroup = (DicomGroup)_groups.elementAt(i);
        while ((dcmGroup.group() != sGroup) && (++i < iGroups))
        {
            dcmGroup = (DicomGroup)_groups.elementAt(i);
            iOffset += dcmGroup.getLength();
        }
        if (i == iGroups)
        {
            throw new DicomGroupNotFoundException("Group "+Dicom.toHexString(sGroup)+" not found");
        }
        
        return dcmGroup.getOffset();
    }

    public short getValue(int iGroup, int iElement, short sVR)
        throws DicomGroupNotFoundException, DicomElementNotFoundException
    {
        DicomGroup dcmGroup;
        int i, iGroups;
        short sGroup, sElement;
        short sValue;
        
        sGroup = (short)iGroup;
        sElement = (short)iElement;
        
        iGroups = _groups.size();
        i = 0;
        dcmGroup = (DicomGroup)_groups.elementAt(i);
        while ((dcmGroup.group() != sGroup) && (++i < iGroups))
        {
            dcmGroup = (DicomGroup)_groups.elementAt(i);
        }
        if (i == iGroups)
        {
            throw new DicomGroupNotFoundException("Group "+Dicom.toHexString(sGroup)+" not found");
        }
        
        try
        {
            sValue = dcmGroup.getValue(sElement, sVR);
        }
        catch (DicomElementNotFoundException exDCM)
        {
            sValue = -1;
            throw new DicomElementNotFoundException("Element "+Dicom.toHexString(sElement)+" not found");
        }
        
        return sValue;
    }

    public int getValue(int iGroup, int iElement, int iVR)
        throws DicomGroupNotFoundException, DicomElementNotFoundException
    {
        DicomGroup dcmGroup;
        int i, iGroups;
        short sGroup, sElement;
        int iValue;
        
        sGroup = (short)iGroup;
        sElement = (short)iElement;
        
        iGroups = _groups.size();
        i = 0;
        dcmGroup = (DicomGroup)_groups.elementAt(i);
        while ((dcmGroup.group() != sGroup) && (++i < iGroups))
        {
            dcmGroup = (DicomGroup)_groups.elementAt(i);
        }
        if (i == iGroups)
        {
            throw new DicomGroupNotFoundException("Group "+Dicom.toHexString(sGroup)+" not found");
        }
        
        try
        {
            iValue = dcmGroup.getValue(sElement, iVR);
        }
        catch (DicomElementNotFoundException exDCM)
        {
            iValue = -1;
            throw new DicomElementNotFoundException("Element "+Dicom.toHexString(sElement)+" not found");
        }
        
        return iValue;
    }

    public String getValue(int iGroup, int iElement, String strVR)
        throws DicomGroupNotFoundException, DicomElementNotFoundException
    {
        DicomGroup dcmGroup;
        int i, iGroups;
        short sGroup, sElement;
        String strValue;
        
        sGroup = (short)iGroup;
        sElement = (short)iElement;
        
        iGroups = _groups.size();
        i = 0;
        dcmGroup = (DicomGroup)_groups.elementAt(i);
        while ((dcmGroup.group() != sGroup) && (++i < iGroups))
        {
            dcmGroup = (DicomGroup)_groups.elementAt(i);
        }
        if (i == iGroups)
        {
            throw new DicomGroupNotFoundException("Group "+Dicom.toHexString(sGroup)+" not found");
        }
        
        try
        {
            strValue = dcmGroup.getValue(sElement, strVR);
        }
        catch (DicomElementNotFoundException exDCM)
        {
            strValue = "";
            throw new DicomElementNotFoundException("Element "+Dicom.toHexString(sElement)+" not found");
        }
        
        return strValue;
    }
    
    public int getXFerSyntax()
    {
        return _xferSyntax;
    }

    public int read(String fileName) throws IOException
    {
        File fin;
        FileInputStream inFile;
        DicomGroup dcmGroup;
        String strSyntax;
        int iOffset;
        byte[] bFileData;

        // debug stuff - needs proper setup
        _filePath = ".";
        _fileName = fileName;
        
        // Read image of file into _data
        fin = new File(_fileName);
        
        // Latch onto the file and check it's actually a DICOM file
        inFile = new FileInputStream(fin);
        _dataLen = (int)fin.length();
        bFileData = new byte[_dataLen];
        inFile.read(bFileData, 0, _dataLen);
        _prefix = Dicom.getString (bFileData, Dicom.PREAMBLE_LEN, Dicom.PREFIX_LEN);
        if (!_prefix.equals(Dicom.PREFIX))
        {
            // Not a DICOM file
            throw new IOException("Not a DICOM file");
        }
                
        // Get DICOM metadata
        iOffset = Dicom.PREAMBLE_LEN + Dicom.PREFIX_LEN;
        dcmGroup = new DicomGroup(bFileData, iOffset);
        try
        {
            strSyntax = dcmGroup.getValue((short)0x0010, Dicom.UI);
        }
        catch (DicomElementNotFoundException exDCM)
        {
            strSyntax = exDCM.toString();
        }
        if (strSyntax.equals(Dicom.IMP_VR_LITTLE_ENDIAN))
        {
            _xferSyntax = Dicom._IMP_VR_LITTLE_ENDIAN;
        }
        else if (strSyntax.equals(Dicom.EXP_VR_LITTLE_ENDIAN))
        {
            _xferSyntax = Dicom._EXP_VR_LITTLE_ENDIAN;
        }
        else if (strSyntax.equals(Dicom.EXP_VR_BIG_ENDIAN))
        {
            _xferSyntax = Dicom._EXP_VR_BIG_ENDIAN;
        }
        else
        {
            _xferSyntax = Dicom._VR_UNKNOWN;
            System.out.println("Error: Unknown or unsupported transfer syntax: "+strSyntax+"!");
        }

        _groups.add(dcmGroup);
        
        iOffset += dcmGroup.getLength();
        while (iOffset < _dataLen)
        {
            dcmGroup = new DicomGroup(bFileData, iOffset, _xferSyntax);
            iOffset += dcmGroup.getLength();
            _groups.add(dcmGroup);
        }
        
        return 0;
    }
    
}