/**
 * Copyright (c) 2003, James d'Arcy <james.darcy@wraithbane.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *    * Redistributions of source code must retain the above copyright notice,
 *      this list of conditions and the following disclaimer.
 *    * Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *    * Neither the name of wraithbane.com nor the names of its contributors
 *      may be used to endorse or promote products derived from this software
 *      without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package com.wraithbane.dicom;

import java.util.Vector;

public class DicomGroup {

    protected short _group;
    protected int _length;
    protected int _start;
    protected int _xferSyntax;
    protected int _offset;
    protected Vector _elements;

    /** Creates a new instance of DicomGroup */
    DicomGroup(byte[] bData, int iOffset)
    {
        _offset = iOffset;
        int iPos = iOffset;
        
        // No xferSyntax in constructor -> Must be Group 0x0002 (MetaData)
        _xferSyntax = Dicom._EXP_VR_LITTLE_ENDIAN;
        
        // Create new Vector to contain the fields of the group
        _elements = new Vector(1, 1);

        // Load 'em up!
        load (bData, iOffset);
    }
    
    /** Creates a new instance of DicomGroup */
    DicomGroup(byte[] bData, int iOffset, int xferSyntax)
    {
        _offset = iOffset;
        int iPos = iOffset;
        
        // Set the transfer syntax
        _xferSyntax = xferSyntax;
        
        // Create new Vector to contain the fields of the group
        _elements = new Vector(1, 1);

        // Load 'em up!
        load (bData, iOffset);
    }
    
    short[] getArrValue(short sTag, short sVR) throws DicomElementNotFoundException
    {
        DicomElement dcmElement;
        int i, iElements;
        
        iElements = _elements.size();
        i = 0;
        dcmElement = (DicomElement)_elements.elementAt(i);
        while ((dcmElement.getTag() != sTag) && (++i < iElements))
        {
            dcmElement = (DicomElement)_elements.elementAt(i);
        }
        if (i == iElements)
        {
            throw new DicomElementNotFoundException("Element "+Dicom.toHexString(sTag)+" not found");
        }
        
        return dcmElement.getArrValue(sVR);
    }
    
    protected int getElementIndex(short sTag)
    {
        DicomElement dcmElement;
        int i;
        
        i = 0;
        dcmElement = (DicomElement)_elements.elementAt(i);
        while (dcmElement.getTag() != sTag)
        {
            i++;
            dcmElement = (DicomElement)_elements.elementAt(i);
        }
        
        return i;
    }
    
    public int getOffset()
    {
        return _offset;
    }
    
    public int getOffsetToElement(short sTag) throws DicomElementNotFoundException
    {
        DicomElement dcmElement;
        int i, iElements, iOffset;
        
        iElements = _elements.size();
        i = 0;
        dcmElement = (DicomElement)_elements.elementAt(i);
        while ((dcmElement.getTag() != sTag) && (++i < iElements))
        {
            dcmElement = (DicomElement)_elements.elementAt(i);
        }
        if (i == iElements)
        {
            throw new DicomElementNotFoundException("Element "+Dicom.toHexString(sTag)+" not found");
        }
        
        return dcmElement.getOffset();
    }
    
    byte getValue(short sTag, byte bVR) throws DicomElementNotFoundException
    {
        DicomElement dcmElement;
        int i, iElements;
        
        iElements = _elements.size();
        i = 0;
        dcmElement = (DicomElement)_elements.elementAt(i);
        while ((dcmElement.getTag() != sTag) && (++i < iElements))
        {
            dcmElement = (DicomElement)_elements.elementAt(i);
        }
        if (i == iElements)
        {
            throw new DicomElementNotFoundException("Element "+Dicom.toHexString(sTag)+" not found");
        }
        
        return dcmElement.getValue(bVR);
    }
    
    short getValue(short sTag, short sVR) throws DicomElementNotFoundException
    {
        DicomElement dcmElement;
        int i, iElements;
        
        iElements = _elements.size();
        i = 0;
        dcmElement = (DicomElement)_elements.elementAt(i);
        while ((dcmElement.getTag() != sTag) && (++i < iElements))
        {
            dcmElement = (DicomElement)_elements.elementAt(i);
        }
        if (i == iElements)
        {
            throw new DicomElementNotFoundException("Element "+Dicom.toHexString(sTag)+" not found");
        }
        
        return dcmElement.getValue(sVR);
    }
    
    int getValue(short sTag, int iVR) throws DicomElementNotFoundException
    {
        DicomElement dcmElement;
        int i, iElements;
        
        iElements = _elements.size();
        i = 0;
        dcmElement = (DicomElement)_elements.elementAt(i);
        while ((dcmElement.getTag() != sTag) && (++i < iElements))
        {
            dcmElement = (DicomElement)_elements.elementAt(i);
        }
        if (i == iElements)
        {
            throw new DicomElementNotFoundException("Element "+Dicom.toHexString(sTag)+" not found");
        }
        
        return dcmElement.getValue(iVR);
    }
    
    String getValue(short sTag, String strVR) throws DicomElementNotFoundException
    {
        DicomElement dcmElement;
        int i, iElements;
        
        iElements = _elements.size();
        i = 0;
        dcmElement = (DicomElement)_elements.elementAt(i);
        while ((dcmElement.getTag() != sTag) && (++i < iElements))
        {
            dcmElement = (DicomElement)_elements.elementAt(i);
        }
        if (i == iElements)
        {
            throw new DicomElementNotFoundException("Element "+Dicom.toHexString(sTag)+" not found");
        }
        
        return dcmElement.getValue(strVR);
    }
    
    short group()
    {
        return _group;
    }
    
    protected int load(byte[] bData, int iOffset)
    {
        int start = iOffset;
        DicomElement element;
        
        // First Field - group length
        element = new DicomElement(bData, iOffset, _xferSyntax);
        _length = element.getValue(Dicom.UL)+12;
        _group = element.getGroup();

        // Store the element
        _elements.add(element);
        
//        System.out.print("Loading group "+Dicom.toHexString(_group)+" ");
        // Loop through the data until we hit the end of the group
        iOffset += element.getLength();
        while (iOffset < start+_length)
        {
            element = new DicomElement(bData, iOffset, _xferSyntax);
            iOffset += element.getLength();
            _elements.add(element);
        }
        
//        System.out.println("- OK");
        
        return 0;
    }

    public int getLength()
    {
        return _length;
    }

}
